package state;

import java.util.Date;

public class Subscription {

    private final float planValue;

    private Date dueDate;

    private boolean lateInterest;
    private float negotiatedInterest;
    private Date suspensionDate;
    private Date today;

    private static Situation active;
    private static Situation late;
    private static Situation recovering;
    private static Situation overdue;
    private static Situation suspended;

    // No construtor, são definidos o valor da assinatura e a data de vencimento do pagamento.
    public Subscription(float planValue, Date dueDay){
        this.dueDate = dueDay;
        this.planValue = planValue;

        this.lateInterest = false;
        this.negotiatedInterest = 0;
        this.suspensionDate = null;
        this.today = null;
    }

    public static void statesSetup(
            Situation _active,
            Situation _late,
            Situation _recovering,
            Situation _overdue,
            Situation _suspended
    ) {
        active = _active;
        late = _late;
        recovering = _recovering;
        overdue = _overdue;
        suspended = _suspended;
    }

    public boolean sendContent(){
        return verifyPayment() == active;
    }

    // Modificadores de situação
    public void activateSubscription() {
        this.suspensionDate = null;
        this.today = null;
        this.lateInterest = false;
        this.negotiatedInterest = 0;
    }

    public void setSubscriptionLate() {
        this.suspensionDate = null;
        this.today = null;
        this.lateInterest = true;
        this.negotiatedInterest = 0;
    }

    public void setSubscriptionRecovering(float negotiatedInterest){
        this.suspensionDate = null;
        this.today = null;
        this.lateInterest = false;
        this.negotiatedInterest = negotiatedInterest;
    }

    public void setSubscriptionOverdue(Date today) {
        this.today = today;
        this.suspensionDate = null;
        this.lateInterest = false;
        this.negotiatedInterest = 0;
    }

    public void suspendSubscription(Date today){
        this.suspensionDate = today;
        this.today = null;
        this.lateInterest = false;
        this.negotiatedInterest = 0;
    }

    private Situation verifyPayment(){
        if(this.lateInterest)
            return late;
        if(this.negotiatedInterest != 0)
            return recovering;
        if(this.suspensionDate!=null)
            return suspended;
        if(this.today!=null)
            return overdue;
        return active;
    }

    public float calculateSubscriptionValue() {
        Situation situationSubscription = this.verifyPayment();
        return situationSubscription.calculateSubscriptionValue(this);
    }


    // Getters & Setters

    public float getPlanValue() {
        return planValue;
    }

    public float getNegotiatedInterest() {
        return negotiatedInterest;
    }

    public Date getSuspensionDate() {
        return suspensionDate;
    }

    public void setSuspensionDate(Date suspensionDate) {
        this.suspensionDate = suspensionDate;
    }

    public Date getDueDate() {
        return dueDate;
    }

    public void setDueDate(Date dueDate) {
        this.dueDate = dueDate;
    }

    public Date getToday() {
        return today;
    }

    public void setToday(Date today) {
        this.today = today;
    }


}